% European option call price using local volatility and PDE solver
clc; clear;

% Market maturities, strikes, implied vols
T  = xlsread('SPY options LV and IV.xls','T');
K  = xlsread('SPY options LV and IV.xls','K');
IV = xlsread('SPY options LV and IV.xls','MktIV');

% Interpolated Local Volatility, times, prices
LVi = xlsread('SPY options LV and IV.xls','LV');  % Rows: Strikes, Col
Ti  = xlsread('SPY options LV and IV.xls','Ti');
Ki  = xlsread('SPY options LV and IV.xls','Ki');

% SPY settings
S0 = 137.14;
r = 0.001;
q = 0.0068;
Strike = 135;
Mat = 7/12;

% Interpolate to get the K/T implied volatility for Black Scholes
sigma = interp2(T,K,IV,Mat,Strike);

% Black Scholes call
BSC = @(s,K,rf,q,v,T) (s*exp(-q*T)*normcdf((log(s/K) + (rf-q+v^2/2)*T)/v/sqrt(T)) - K*exp(-rf*T)*normcdf((log(s/K) + (rf-q+v^2/2)*T)/v/sqrt(T) - v*sqrt(T)));
BlackScholesPrice = BSC(S0,Strike,r,q,sigma,Mat);


%% Grid for the stock price
nS = 40;
nT = 100;
NS = nS+1;
NT = nT+1;
GridType = 'Uniform';
Tmin = 0; 
Tmax = Mat;
Smax = 1.75*Strike;

if strcmp(GridType,'Uniform')
    Smin = 0.50*Strike;
    ds = (Smax-Smin)/nS;
    S = Smin:ds:Smax;
else
    c = Strike/30;
    dz = 1/nS*(asinh((Smax-Strike)/c) - asinh(-Strike/c));
    for i=1:nS+1;
        z(i) = asinh(-Strike/c) + (i-1)*dz;
        S(i) = Strike + c*sinh(z(i));
    end
end

% Maturity grid -- common to both uniform and non-uniform grid types
dt = (Tmax-Tmin)/nT;
T = [0:NT-1].*dt;

%% Extrapolate the Local volatility surface with T=0
DK = Ki(2)-Ki(1);
DT = Ti(2)-Ti(1);
Ki2 = [S(1) Ki S(end)];
Ti2 = [0 Ti];
TopRow = LVi(1,:);
BotRow = LVi(end,:);
LeftCol = [LVi(1,1); LVi(:,1); LVi(end,1)];
LVi2 = [TopRow; LVi; BotRow];
LVi2 = [LeftCol LVi2];


%% Identity matrix for the A and B matrices
I = eye(NS*NS);

% Explicit, Implicit, and Crank-Nicolson prices
thet = [0 1 0.5];
for k=1:length(thet)
    Price(k) = WeightedMethodBS(thet(k),S0,Strike,r,q,S,T,LVi2,Ti2,Ki2,dt,GridType)
end
Error = BlackScholesPrice - Price;


%% Output everything
fprintf('----------------------------------------------------------\n');
disp([GridType ' grid type'])
fprintf('Stock price grid size of %5.0f \n',NS);
fprintf('Time step grid size of   %5.0f \n',NT);
fprintf('----------------------------------------------------------\n');
fprintf('Method                     Price        Dollar Difference \n');
fprintf('----------------------------------------------------------\n');
fprintf('Black Scholes        %12.4f \n',BlackScholesPrice);
fprintf('Explicit method      %12.4f   %12.4f \n',Price(1),Error(1));
fprintf('Implicit method      %12.4f   %12.4f \n',Price(2),Error(2));
fprintf('Crank-Nicolson       %12.4f   %12.4f \n',Price(3),Error(3));
fprintf('----------------------------------------------------------\n');


