function WPrice = WeightedMethodBS(thet,S0,Strike,r,q,S,T,LVi,Ti,Ki,dt,GridType)

% Heston Call price using the Weighted Method
% Requires a uniform grid for the stock price, volatility, and maturity
% INPUTS
%   thet = Weighing parameter
%   S0 = Spot price at which to price the call
%   V0 = variance at which to price the call
%   K  = Strike price
%   S = uniform grid for the stock price
%   V = uniform grid for the volatility
%   T = uniform grid for the maturity
%   A = "A" matrix for weighted method
%   invA = A inverse
%   B = "B" matrix for weighted method
% OUTPUT
%   y = 2-D interpolated European Call price

% Required vector lengths and time increment
NS = length(S);
NT = length(T);

% Initialize the u vector
u = zeros(NS*NS,1);

% U(0) vector - value of U(T) at maturity
Si = repmat(S',NS,1);
U = max(0, Si - Strike);

% Obtain the derivatives matrices
if strcmp(GridType,'Uniform')
    [derS derSS R] = BuildDerivativesUniformBS(S,T);
else
    [derS derSS R] = BuildDerivativesNonUniformBS(S,T);
end
I = eye(NS*NS);

% Loop through the time increments, updating U(t) to U(t+1) at each step
for t=2:NT
	u = U;
    T0 = T(t);
    % Interpolate the local volatility and create a local vol matrix
    volvec = interp2(Ti,Ki',LVi,T0,S);
    volmat = repmat(volvec,1,NS);
    sigma = repmat(volmat,NS,NS);
    % The L operator matrix
    L = (r-q).*derS + (0.5.*sigma.^2).*derSS - r.*R;
    % Matrices for weighted method
    A = (I - thet.*dt.*L);
    B = (I + (1-thet).*dt.*L);
    % Inverse of A
    if thet == 0;
        invA = I;
    else
        invA = inv(A);
    end
	if thet==0
		U = B*u;         % Explicit Method
	elseif thet==1
		U = invA*u;      % Implicit Method
	else
		U = A\B*u;       % Weighted Method
	end
end

% Retain only the first NS terms of the U vector
U1 = U(1:NS);

% Interpolate to get the final price
WPrice = interp1(S',U1,S0);
